#!/usr/bin/perl -w

######################################################
#
#	Example of wrapper exploration
#
#	This script explore impact of ambient temperature
#	and produce a normalized TCO and a normalized environmental impact
#	for each configuration in a file $filename.txt
#
######################################################

use strict;

######################################################
#
# $basedir VARIABLE TO MODIFY if you change the 
# organization of directories
#
# put the basedir of the tco enterprise tool in $basedir
# example: my $basedir="/home/user/tco/enterprise/";
#
######################################################

my $basedir="../../";

#to get the current dir
my $currentdir=`pwd`;
chomp $currentdir;

######################################################
#
# The wrapper exploration start here
#
######################################################

#output file's name  without extension
my $filename="ambient";

#legende
my $legende="#temperature \t normalized TCO \t normalized env impact\n";


#open outputfile and print the legende
open(OUTPUTFILE,">$currentdir/output/$filename.txt") || die ("Error when opening $currentdir/output/$filename.txt") ;
print(OUTPUTFILE "$legende");

#goto the main directory to call the enterprise tool
if($basedir eq "")
{
	print("Please, set the \$basedir variable before using this script\n");
	exit(0);
}
chdir($basedir) or die "$!";


####################################################
# main exploration loop
####################################################

my $first=1;
my $tco_ref=0;
my $co2_ref=0;

my @temperature=("20","21","22","23","24","25","26","27","28","29","30");

my $temperature_ref=20;
my $pue_ref=2;


my $pue_coef= (2-1.65)/10;

#Acceleration factor
my $k=8.617e-05;
my $Ea=0.3;
####

foreach my $temperature_current (@temperature)
{
	
	my $current_pue = $pue_ref - ($temperature_current - $temperature_ref) * $pue_coef ;
	
	my $AF=exp( ($Ea/$k) * ( (1/($temperature_ref+273)) -  (1/($temperature_current+273)) ) );
	
	my $current_mttf_coef = 1/$AF;
	
	#clean the previous configuration file
	if (-e "$currentdir/s.params") 
	{
		system("rm $currentdir/s.params");
	}
	if (-e "$currentdir/dc.params") 
	{
		system("rm $currentdir/dc.params");
	}
	
	#generate the current configuration file
	system("cat $currentdir/template/dc_template.params | sed -e 's/XXPUE/$current_pue/g' > $currentdir/dc.params");
	system("cat $currentdir/template/s_template.params | sed -e 's/XXMTTF_COEF/$current_mttf_coef/g'  > $currentdir/s.params");
	
	#call the enterprise tool and keep the output
	system("./eetco.pl $currentdir/dc.params $currentdir/s.params  > $currentdir/output/$temperature_current.res");
	
	#get the TCO value
	my $tco=`cat $currentdir/output/$temperature_current.res | grep tco | gawk '{print \$2}'`;
	chomp $tco;
	my $co2=`cat $currentdir/output/$temperature_current.res | grep co2peryr | gawk '{print \$2}'`;
	chomp $co2;
	
	if($first==1) #if it is the first iteration, keep the referrence values to normalized
	{
		$first=0;
		$tco_ref=$tco;
		$co2_ref=$co2;
	}
	
	my $normalized_tco=$tco/$tco_ref;
	
	my $normalized_co2 = $co2/$co2_ref;
	
	print(OUTPUTFILE "$temperature_current \t;\t  $normalized_tco \t;\t $normalized_co2\n");
}

close(OUTPUTFILE);

#keep the output file in the current directory
system("cp $currentdir/output/$filename.txt $currentdir/$filename.txt");

#go back to the current dir
chdir($currentdir) or die "$!";

##################################

####################################################
# gnulpot call
####################################################

#if (-e "./plot.plot") 
#{
#	system("rm ./plot.plot");
#}	
#
#system("cat ./template/gnuplot_template.plot | sed -e 's/XXNAME/$filename/g' > ./plot.plot");
#
#system("gnuplot <plot.plot");


####################################################
# clean
####################################################
#system("rm ./plot.plot");
system("rm ./dc.params");
system("rm ./s.params");
system("rm ./output/*");
#system("rm $currentdir/$filename.txt");
